// Qt
#include <QFileInfo>
#include <QTime>
#include <QTimer>
#include <QDir>
#include <QStringList>

//GDCM
#include <gdcmImageReader.h>

#include <iostream>

#include <vtkGDCMImageReader.h>
#include <vtkSmartPointer.h>
#include <vtkStringArray.h>
#include <itkImageSeriesReader.h>

void readingWithVTKGDCM(const QStringList &filenames)
{
    if (filenames.isEmpty())
    {
        return;
    }

    vtkSmartPointer<vtkGDCMImageReader> vtkGDCMReader = vtkSmartPointer<vtkGDCMImageReader>::New();
    vtkGDCMReader->LoadIconImageOff();
    vtkGDCMReader->LoadOverlaysOff();
    if (filenames.size() > 1)
    {
        vtkSmartPointer<vtkStringArray> stringArray = vtkSmartPointer<vtkStringArray>::New();
        foreach (const QString &filename, filenames)
        {
            stringArray->InsertNextValue(filename.toStdString());
        }

        vtkGDCMReader->SetFileNames(stringArray);
    }
    else
    {

        vtkGDCMReader->SetFileName(qPrintable(filenames.first()));
    }

    vtkGDCMReader->Update();
}

void readingWithITKGDCM(const QStringList &filenames)
{
    itk::ImageSeriesReader<itk::Image<unsigned short int, 3> >::Pointer itkSeriesReader = itk::ImageSeriesReader<itk::Image<unsigned short int, 3> >::New();
    std::vector<std::string> stlFilenames;
    foreach (const QString &filename, filenames)
    {
        stlFilenames.push_back(filename.toStdString());
    }
    itkSeriesReader->SetFileNames(stlFilenames);
    itkSeriesReader->Update();
}

int main(int argc, char *argv[])
{
    if (argc < 2)
    {
        std::cout << "Usage: " << argv[0] << " <directory> " << std::endl;
        return -1;
    }

    QStringList filenames;

    QDir dir(argv[1]);
    QFileInfoList fileInfoList = dir.entryInfoList(QDir::Files);

    foreach (const QFileInfo &fileInfo, fileInfoList)
    {
        gdcm::ImageReader reader;
        reader.SetFileName( fileInfo.absoluteFilePath().toLatin1());
        if (reader.Read())
        {
            filenames << fileInfo.absoluteFilePath();
        }
    }

    QTime time;
    time.start();
    readingWithVTKGDCM(filenames);
    std::cout << "Time reading with VTKGDCM: " << time.elapsed() << " ms" << std::endl;

    time.restart();
    readingWithITKGDCM(filenames);
    std::cout << "Time reading with ITKGDCM: " << time.elapsed() << " ms" << std::endl;

    return 0;
}
